---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons
local gfrm = Glance.Frames

---------------------------
-- create the button
---------------------------
gf.AddButton("Pets","RIGHT")
local btn = gb.Pets
btn.text              = "      "
btn.enabled           = true
btn.texture.normal    = "Interface\\AddOns\\Glance_Pets\\paw.tga"
btn.texture.favorite  = "Interface\\AddOns\\Glance_Pets\\favorites.tga"
btn.texture.filter    = "Interface\\ChatFrame\\ChatFrameExpandArrow"
btn.texture.health    = "Interface\\AddOns\\Glance_Pets\\health.tga"
btn.texture.power     = "Interface\\AddOns\\Glance_Pets\\power.tga"
btn.texture.speed     = "Interface\\AddOns\\Glance_Pets\\speed.tga"
btn.texture.type      = "Interface\\AddOns\\Glance_Pets\\"
btn.events            = {"COMPANION_LEARNED","COMPANION_UPDATE","PET_JOURNAL_AUTO_SLOTTED_PET","PET_JOURNAL_LIST_UPDATE","PET_JOURNAL_PET_DELETED"}
btn.update            = true
btn.timer1            = true
btn.click             = true
btn.tooltip           = true
btn.menu              = false
btn.save.perAccount   = {}

---------------------------
-- shortcuts
---------------------------
local HEX = ga.colors.HEX
local QA = ga.colors.QUALITY
local QARGB = ga.colors.QUALITYRGB
local tooltip = gf.Tooltip

---------------------------
-- variables
---------------------------
local totalCount, seqtime, position = 0,0,0
local modelWidth = 235
local petHeight = 400
local petWidth = 230
local iconSize = 24
local nameLengthLimit = 18
local hurtCount = 0
local searchText = ""

---------------------------
-- arrays
---------------------------
ga.PetQualityCount = {
	[1] = 0,
	[2] = 0,
	[3] = 0,
	[4] = 0,
	[5] = 0,
	[6] = 0,
}
ga.PetQuality = {
	[1] = "Poor",
	[2] = "Common",
	[3] = "Uncommon",
	[4] = "Rare",
	[5] = "Epic",
	[6] = "Legendary",
}
ga.petBreed = {
	["B/B"] = "Balance",			--[3]  {.5,.5,.5}
	["P/P"] = "Destruction",		--[4]  {0,2,0}
	["S/S"] = "Ninja",				--[5]  {0,0,2}
	["H/H"] = "Guardian",			--[6]  {2,0,0}
	["H/P"] = "Power & Health",		--[7]  {.9,.9,0}
	["P/S"] = "Power & Speed",		--[8]  {0,.9,.9}
	["H/S"] = "Health & Speed",		--[9]  {.9,0,.9}
	["P/B"] = "Balanced Power",		--[10] {.4,.9,.4}
	["S/B"] = "Balanced Speed",		--[11] {.4,.4,.9}
	["H/B"] = "Balanced Health",	--[12] {.9,.4,.4}
}

---------------------------
-- EasyMenu redirect
---------------------------
function gf.Pets.EasyMenu()
	Glance.Debug("function","EasyMenu","Pets")
	UIDropDownMenu_Initialize(gfrm.petFrameFilterMenu, gf.Pets.Filter_Initialize, "MENU");
	ToggleDropDownMenu(1, nil, gfrm.petFrameFilterMenu, gfrm.petFrameFilter, 0, 0);
end

---------------------------
-- Update Filter Table
---------------------------
function gf.Pets.Filter_Initialize(self,level)
	Glance.Debug("function","Filter_Initialize","Pets")
    local info = UIDropDownMenu_CreateInfo();
	
	-- default
    info.keepShownOnClick = true;   
 
	-- level 1
    if level == 1 then
		-- level 1 defaults
        info.checked =  nil;
        info.isNotRadio = nil;
        info.func =  nil;
        info.hasArrow = true;
        info.notCheckable = true;
        
		-- filter by family
        info.text = PET_FAMILIES
        info.value = 1;
        UIDropDownMenu_AddButton(info, level)
         
		-- filter by source
        info.text = SOURCES
        info.value = 2;
        UIDropDownMenu_AddButton(info, level)
        
		-- sort list
        info.text = RAID_FRAME_SORT_LABEL
        info.value = 3;
        UIDropDownMenu_AddButton(info, level)
     
	-- level 2
    else 	
		-- defaults
		info.hasArrow = false;
		info.isNotRadio = true;
		info.notCheckable = true;
		
		-- filter by family
        if UIDROPDOWNMENU_MENU_VALUE == 1 then
			----------------------
            -- check all         
            info.text = CHECK_ALL
            info.func = function()
                            C_PetJournal.AddAllPetTypesFilter();
                            UIDropDownMenu_Refresh(gfrm.petFrameFilterMenu, 1, 2);
                        end
            UIDropDownMenu_AddButton(info, level)             
			-- check none
            info.text = UNCHECK_ALL
            info.func = function()
                            C_PetJournal.ClearAllPetTypesFilter();
                            UIDropDownMenu_Refresh(gfrm.petFrameFilterMenu, 1, 2);
                        end
            UIDropDownMenu_AddButton(info, level)
			----------------------
			-- iterate families
            info.notCheckable = false;
            local numTypes = C_PetJournal.GetNumPetTypes();
            for i=1,numTypes do
                info.text = _G["BATTLE_PET_NAME_"..i];
                info.func = function(_, _, _, value)
                            C_PetJournal.SetPetTypeFilter(i, value);
                            UIDropDownMenu_Refresh(gfrm.petFrameFilterMenu, 1, 2);
							gf.Pets.applyFilters();
                        end
                info.checked = function() return not C_PetJournal.IsPetTypeFiltered(i) end;
                UIDropDownMenu_AddButton(info, level);
            end
		-- filter by source
        elseif UIDROPDOWNMENU_MENU_VALUE == 2 then
			----------------------
			-- check all
            info.text = CHECK_ALL
            info.func = function()
                            C_PetJournal.AddAllPetSourcesFilter();
                            UIDropDownMenu_Refresh(gfrm.petFrameFilterMenu, 2, 2);
                        end
            UIDropDownMenu_AddButton(info, level)
            -- check none
            info.text = UNCHECK_ALL
            info.func = function()
                            C_PetJournal.ClearAllPetSourcesFilter();
                            UIDropDownMenu_Refresh(gfrm.petFrameFilterMenu, 2, 2);
                        end
            UIDropDownMenu_AddButton(info, level)
			----------------------         
			-- iterate sources
            info.notCheckable = false;
            local numSources = C_PetJournal.GetNumPetSources();
            for i=1,numSources do
                info.text = _G["BATTLE_PET_SOURCE_"..i];
                info.func = function(_, _, _, value)
                            C_PetJournal.SetPetSourceFilter(i, value);
                            UIDropDownMenu_Refresh(gfrm.petFrameFilterMenu, 2, 2);
							gf.Pets.applyFilters()
                        end
                info.checked = function() return not C_PetJournal.IsPetSourceFiltered(i) end;
                UIDropDownMenu_AddButton(info, level);
            end
		-- sort list
        elseif UIDROPDOWNMENU_MENU_VALUE == 3 then
			-- change defaults
            info.isNotRadio = nil;
            info.notCheckable = nil;
            info.keepShownOnClick = nil;    
            -- sort by name
            info.text = NAME
            info.func = function()
                            C_PetJournal.SetPetSortParameter(LE_SORT_BY_NAME);
                            gf.Pets.applyFilters()
                        end
            info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_NAME end;
            UIDropDownMenu_AddButton(info, level);
            -- sort by level
            info.text = LEVEL
            info.func = function()
                            C_PetJournal.SetPetSortParameter(LE_SORT_BY_LEVEL);
                            gf.Pets.applyFilters()
                        end
            info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_LEVEL end;
            UIDropDownMenu_AddButton(info, level);
            -- sort by rarity
            info.text = RARITY
            info.func = function()
                            C_PetJournal.SetPetSortParameter(LE_SORT_BY_RARITY);
                            gf.Pets.applyFilters()
                        end
            info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_RARITY end;
            UIDropDownMenu_AddButton(info, level);
            --sort by type
            info.text = TYPE
            info.func = function()
                            C_PetJournal.SetPetSortParameter(LE_SORT_BY_PETTYPE);
                            gf.Pets.applyFilters()
                        end
            info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_PETTYPE end;
            UIDropDownMenu_AddButton(info, level);
        end
    end
end

---------------------------
-- update
---------------------------
function gf.Pets.update(self, event, arg1)
	Glance.Debug("function","update","Pets")
	if event == "COMPANION_LEARNED" then
		gf.Pets.build()
	elseif event == "COMPANION_UPDATE" or event == "PET_JOURNAL_AUTO_SLOTTED_PET" or event == "PET_JOURNAL_LIST_UPDATE" or event == "PET_JOURNAL_PET_DELETED" then
		gf.Pets.applyFilters()
	else
		-- hide if moving
		if GetUnitSpeed("Player") ~= 0 then
			if gfrm.petFrame == nil then
				return
			else
				if gfrm.petFrame:IsShown() then
					gfrm.petFrame:Hide()
				end
			end
		end
	end
end

---------------------------
-- tooltip
---------------------------
function gf.Pets.tooltip()	
	Glance.Debug("function","tooltip","Pets")
	local numPets, totalPets = C_PetJournal.GetNumPets()
	-- reset tooltip counts
	for i=1,6 do
		ga.PetQualityCount[i]=0
	end
	hurtCount = 0	
	-- update tooltip counts
	for i=1,numPets do
		local rarity = select(9,gf.Pets.petInfo(i))
		ga.PetQualityCount[rarity] = ga.PetQualityCount[rarity] + 1
		if C_PetJournal.PetIsHurt(select(1,C_PetJournal.GetPetInfoByIndex(i, false))) then
			hurtCount = hurtCount + 1
		end
	end
	tooltip.Title("Pets", "GLD")
	tooltip.Wrap("You have obtained a total of "..HEX.green..totalPets..HEX.white.." companion pets.","WHT")
	tooltip.Space()
	tooltip.Line("Pet Journal", "GLD")
	tooltip.Wrap("You are displaying "..HEX.green..numPets..HEX.white.." companion pets.","WHT")
	tooltip.Space()
	tooltip.Line("Quality", "GLD")
	tooltip.Wrap(gf.Pets.getBreakdown(),"GLD")
	tooltip.Space()
	tooltip.Line("Health", "GLD")
	if hurtCount == 0 then
		tooltip.Wrap(hurtCount.."/"..numPets.." are injured.","GRN")
	else
		tooltip.Wrap(hurtCount.."/"..numPets.." are injured.","RED")
	end
	tooltip.Space()
	tooltip.Line("Legend", "GLD")
	tooltip.Line("Pets marked with a star are favorites.","WHT")
	tooltip.Line(HEX.red.."Red |rpets cannot be used as battle pets.","WHT")
	tooltip.Line("Pets are otherwise colored by their "..QA[1].."q"..QA[2].."u"..QA[3].."a"..QA[4].."l"..QA[5].."i"..QA[6].."t|ry.","WHT")
	tooltip.Notes("summon a random pet",nil,"summon a specific pet",nil,"This list is controlled by the filters in your Pet Journal")
end



---------------------------
-- tooltip quality count
---------------------------
function gf.Pets.getBreakdown()
	Glance.Debug("function","getBreakdown","Pets")
	local b = ""
	if ga.PetQualityCount[6] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[6])..QA[6].." "..ga.PetQuality[6]..". " end
	if ga.PetQualityCount[5] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[5])..QA[5].." "..ga.PetQuality[5]..". " end
	if ga.PetQualityCount[4] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[4])..QA[4].." "..ga.PetQuality[4]..". " end
	if ga.PetQualityCount[3] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[3])..QA[3].." "..ga.PetQuality[3]..". " end
	if ga.PetQualityCount[2] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[2])..QA[2].." "..ga.PetQuality[2]..". " end
	if ga.PetQualityCount[1] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[1])..QA[1].." "..ga.PetQuality[1]..". " end
	return b
end

---------------------------
-- click
---------------------------
function gf.Pets.click(self, button, down)
	Glance.Debug("function","click","Pets")
	Glance.Debug("function","click","Pets")
	if button == "LeftButton" then
		C_PetJournal.SummonRandomPet(true);
	elseif button == "RightButton" then
		gf.Pets.build()
		gf.Pets.display()
	end
end

---------------------------
-- is summoned
---------------------------
function gf.Pets.isSummoned(petID)
	Glance.Debug("function","isSummoned","Pets")
	if C_PetJournal.GetSummonedPetGUID() == petID then
		return true
	else
		return false
	end
end

---------------------------
-- get pet info
---------------------------
function gf.Pets.petInfo(id)
	Glance.Debug("function","petInfo","Pets")
	--petID, speciesID, isOwned, customName, level, isFavorite, isRevoked, name, icon, petType, creatureID, cSource, cDescription, isWildPet, canBattle, tradable, unique
	local petID, _,_, customName, _, isFavorite, _, name, icon, petType, _, cSource, cDescription, _, canBattle, _, _ = C_PetJournal.GetPetInfoByIndex(id, false)
	if petID then
		--speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique
		local _, _, level, xp, maxXp, displayID, _ = C_PetJournal.GetPetInfoByPetID(petID)
		--health, maxHealth, attack, speed, rarity
		local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
		return petID, displayID, name, customName, cSource, cDescription, isFavorite, canBattle, rarity, icon, math.floor((health/maxHealth)*100), math.floor((xp/maxXp)*100), level, health, attack, speed, petType
	else
		return nil
	end
end

---------------------------
-- display pet frame
---------------------------
function gf.Pets.display()
	Glance.Debug("function","display","Pets")
	if gfrm.petFrame:IsShown() then
		gfrm.petFrame:Hide()
	else
		gfrm.petFrame:Show()
		gfrm.petDataFrame:Hide()
	end
end

---------------------------
-- pet button mouseover
---------------------------
function gf.Pets.mouseOver(index, petID, displayID, cName, cDescription, cHealth, cXP, cLevel, canBattle, isFavorite, specs, petType)
	Glance.Debug("function","mouseOver","Pets")
	local isSummoned = gf.Pets.isSummoned(petID)
	local m_btn = gf.Pets.mapObjects(index)
	-- update the model
	gf.Pets.ModelUpdate(displayID, cName, cDescription, cHealth, cXP, cLevel, isFavorite, specs, petType);
	-- if usable and not summoned then highlight gold
	if not isSummoned and canBattle then
		m_btn.fs:SetTextColor(.9, .7, .2 ,1);
	end
end

---------------------------
-- pet button mouseout
---------------------------
function gf.Pets.mouseOut(index, petID, canBattle)
	Glance.Debug("function","mouseOut","Pets")
	local isSummoned = gf.Pets.isSummoned(petID)
	local m_btn = gf.Pets.mapObjects(index)
	-- mouseout, turn off the model
	gf.Pets.ModelUpdate();
	-- if usable and not summoned then turn white
	if not isSummoned and canBattle then
		m_btn.fs:SetTextColor(1,1,1,1);
	end		
end

---------------------------
-- pet button mouseclick
---------------------------
function gf.Pets.mouseClick(petID, button)	
	Glance.Debug("function","mouseClick","Pets")
	if button == "LeftButton" then
		-- dismiss or summon
		C_PetJournal.SummonPetByGUID(petID)
	else	
		-- set/unset favorite
		if C_PetJournal.PetIsFavorite(petID) then
			C_PetJournal.SetFavorite(petID, 0)
		else
			C_PetJournal.SetFavorite(petID, 1)
		end
	end
	--reset 
	gf.Pets.applyFilters()
end

---------------------------
-- update model data
---------------------------
function gf.Pets.ModelUpdate(displayID,title,desc, health, xp, level, fav, specs, petType)
	Glance.Debug("function","ModelUpdate","Pets")
	if (displayID == nil) then
		gfrm.petModelFrame:ClearModel()
		gfrm.petDataFrame:Hide()
		gfrm.petModelFrame:Hide()
	else
		gfrm.petModelFrame:SetDisplayInfo(displayID)
		local t, d, h, x, f, l, s, pt = gf.Pets.mapObjects()
		t:SetText(title)
		t:SetHeight(t:GetStringHeight()+15)
		d:SetText(desc)
		d:SetHeight(d:GetStringHeight()+35)
		l:SetText(level)
		s:SetText(specs)
		s:SetHeight(s:GetStringHeight()+35)
		gfrm.petDataFrame:Show()
		gfrm.petModelFrame:Show()
		h:SetValue(health)
		x:SetValue(xp)
		pt:SetTexture(btn.texture.type..petType..".tga")	
		if fav then
			f:SetTexture(btn.texture.favorite)	
			f:SetWidth(16)
		else
			f:SetTexture(0,0,0,0)	
		end
	end
end

---------------------------
-- update pet status
---------------------------
function gf.Pets.applyFilters()
	Glance.Debug("function","applyFilters","Pets")
	-- not built yet, go away
	if gfrm.petFrame == nil then return end
	local numPets, totalPets = C_PetJournal.GetNumPets()
	-- hide all buttons
	for i=1, totalPets do	
		-- link to the proper objects		
		local m_btn = gf.Pets.mapObjects(i)		
		if m_btn ~= nil then
			-- hide all to start
			m_btn:Hide();
		end
	end	
	-- now iterate the number of pets the filter shows
	for i=1, numPets do	
		local petID, displayID, name, customName, cSource, cDescription, isFavorite, canBattle, rarity, icon, cHealth, cXP, cLevel, cHealth, cPower, cSpeed, petType = gf.Pets.petInfo(i)
		local breedString = QA[rarity]..ga.PetQuality[rarity].." |r"
		
		-- show the breed if possible
		if IsAddOnLoaded("BattlePetBreedID") then
			breedString = breedString..QA[rarity]..ga.petBreed[GetBreedID_Journal(petID)].." ("..GetBreedID_Journal(petID)..")|r"
		end
		-- health
		breedString = breedString.."\n|T"..btn.texture.health..":16|t "..cHealth
		-- power
		breedString = breedString.."  |T"..btn.texture.power..":16|t "..cPower
		-- speed
		breedString = breedString.."  |T"..btn.texture.speed..":16|t "..cSpeed.."\n\n"
		
		-- link to the proper objects
		local m_btn = gf.Pets.mapObjects(i)	
		
		-- if the button exists
		if m_btn ~= nil then					
		
			-- set everything white and no icon
			m_btn.fs:SetTextColor(1,1,1,1)
			m_btn.favtexture:SetTexture(0,0,0,0)
			
			-- set rarity
			local r,g,b = unpack(QARGB[rarity])
			m_btn.usetexture:SetTexture(r,g,b,1)
			
			-- set text (retain a decent width)
			local bName = name
			if string.len(bName) > nameLengthLimit then
				bName = string.sub(bName,1,nameLengthLimit)..".."
			end
			-- set the level text
			if string.len(cLevel) == 1 then
				m_btn.level:SetText("0"..cLevel)
			else
				m_btn.level:SetText(cLevel)
			end
			-- set the breed if available
			if IsAddOnLoaded("BattlePetBreedID") then
				m_btn.breed:SetText("|T"..btn.texture.type..petType..".tga:16|t "..QA[rarity]..ga.petBreed[GetBreedID_Journal(petID)])
			end
					
			-- set the pet name
			m_btn:SetText("|T"..icon..":"..iconSize.."|t "..bName) 
			
			-- reset the width to actual width
			m_btn:SetWidth(m_btn.fs:GetStringWidth())
							
			-- mouse events
			if customName then
				m_btn:SetScript("OnEnter", function(self, button, down) gf.Pets.mouseOver(i, petID, displayID, QA[rarity]..name.."|r\n\""..customName.."\"", cDescription.."\n\n"..cSource.."\n\n", cHealth, cXP, cLevel, canBattle, isFavorite, breedString,petType) end)
			else
				m_btn:SetScript("OnEnter", function(self, button, down) gf.Pets.mouseOver(i, petID, displayID, QA[rarity]..name, cDescription.."\n\n"..cSource.."\n\n", cHealth, cXP, cLevel, canBattle, isFavorite, breedString,petType) end)
			end
			m_btn:SetScript("OnLeave", function(self, button, down) gf.Pets.mouseOut(i, petID, canBattle) end)
			m_btn:SetScript("OnClick", function(self, button, down) gf.Pets.mouseClick(petID, button) end)
			
			-- set favorite icon
			if isFavorite then
				m_btn.favtexture:SetTexture(btn.texture.favorite)	
				m_btn.favtexture:SetWidth(16)
			else
				m_btn.favtexture:SetTexture(0,0,0,0)
			end				
			
			-- set summoned color
			if gf.Pets.isSummoned(petID) then
				m_btn.fs:SetTextColor(.9, .7, .2 ,1)
			end
			
			-- set useable texture
			if not canBattle then
				m_btn.fs:SetTextColor(1,0,0,.7)
				m_btn.usetexture:SetTexture(1,0,0,1)
			end
			
			-- show this one
			m_btn:Show();
		end
	end
end

---------------------------
-- search
---------------------------
function gf.Pets.search(self)
	searchText = self:GetText();
	if searchText == SEARCH then
		C_PetJournal.SetSearchFilter("");
		return;
	end	
	C_PetJournal.SetSearchFilter(searchText);
	gf.Pets.applyFilters()
end

---------------------------
-- build all
---------------------------
function gf.Pets.build()
	Glance.Debug("function","build","Pets")
	gf.Pets.buildFrames()
	gf.Pets.buildButtons()
	gf.Pets.applyFilters()
end

---------------------------
-- build frames
---------------------------
function gf.Pets.buildFrames()
	Glance.Debug("function","buildFrames","Pets")
	if gfrm.petFrame == nil then	
		-------------------------------------------------------------
		-- create main container
		gfrm.petFrame = CreateFrame("FRAME", "Glance_petFrame", Glance_Buttons_Pets, "TooltipBorderedFrameTemplate")
		gfrm.petFrame:SetPoint("TOPRIGHT", Glance_Buttons_Pets, "BOTTOMRIGHT", 0, 0)	
		gfrm.petFrame:SetWidth(petWidth)
		gfrm.petFrame:SetHeight(petHeight)
		gfrm.petFrame:SetAlpha(.9)
		gfrm.petFrame:SetFrameStrata("High")
		gfrm.petFrame:Hide()
		--gfrm.petFrame:SetScript("OnLeave", function(self, button, down) gfrm.petFrame:Hide() end)

		-- create list petScrollFrame
		gfrm.petScrollFrame = CreateFrame("ScrollFrame", "Glance_petScrollFrame", gfrm.petFrame, "UIPanelScrollFrameTemplate")
		gfrm.petScrollFrame:SetPoint("TOPLEFT", gfrm.petFrame, "TOPLEFT", 10, -40)	
		gfrm.petScrollFrame:SetWidth(gfrm.petFrame:GetWidth()-40)
		gfrm.petScrollFrame:SetHeight(gfrm.petFrame:GetHeight()-51)
		gfrm.petScrollFrame:Show()

		-- create list frame
		gfrm.petListFrame = CreateFrame("FRAME", "Glance_petListFrame", gfrm.petScrollFrame)
		gfrm.petListFrame:SetPoint("TOPLEFT", gfrm.petScrollFrame, "TOPLEFT", 0, 0)	
		gfrm.petListFrame:SetWidth(gfrm.petScrollFrame:GetWidth())
		gfrm.petListFrame:SetHeight(gfrm.petScrollFrame:GetHeight())
		gfrm.petListFrame:Show()	

		-- bind the list to the petScrollFrame
		gfrm.petScrollFrame:SetScrollChild(gfrm.petListFrame)
		
		-- create the filter by type button
		gfrm.petFrameFilterMenu = CreateFrame("Frame", "Glance_petFrameFilterMenu", nil, "UIDropDownMenuTemplate")
		gfrm.petFrameFilter = CreateFrame("BUTTON", "Glance_petFrameFilter", gfrm.petFrame, "UIMenuButtonStretchTemplate")
		gfrm.petFrameFilter:SetPoint("TOPLEFT", gfrm.petFrame, "TOPLEFT", 10, -10)	
		gfrm.petFrameFilter:SetWidth(100)
		gfrm.petFrameFilter:SetHeight(30)
		gfrm.petFrameFilter:Show()		
		
		-- create the icon overlay
		gfrm.petFrameFilter.arrowtexture = gfrm.petFrameFilter:CreateTexture("Glance_Pet_Filter_Icon", "OVERLAY")
		gfrm.petFrameFilter.arrowtexture:SetTexture(btn.texture.filter)
		gfrm.petFrameFilter.arrowtexture:SetPoint("RIGHT",-5,0)
		gfrm.petFrameFilter.arrowtexture:SetWidth(12)
		gfrm.petFrameFilter.arrowtexture:SetHeight(10)
		
		-- create the font
		local fs = gfrm.petFrameFilter:CreateFontString("Glance_petFrameFilter_FontString")
		fs:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+7)
		fs:SetJustifyH("LEFT")
		fs:SetJustifyV("CENTER")
		fs:SetPoint("LEFT", 10, 0)
		fs:SetTextColor(.9, .7, .2,1)				
		
		-- create the text
		gfrm.petFrameFilter:SetFontString(fs)
		gfrm.petFrameFilter:SetText("Filters")
		
		-- reset the width to actual width
		gfrm.petFrameFilter:SetWidth(fs:GetStringWidth()+30)
		gfrm.petFrameFilter:SetHeight(fs:GetStringHeight()+10)
		
		-- register for clicks
		gfrm.petFrameFilter:RegisterForClicks("AnyUp")			
		
		-- mouse events
		gfrm.petFrameFilter:SetScript("OnClick", function(self, button, down) gf.Pets.EasyMenu() end)
		
		-- create the close button
		gfrm.petFrameClose = CreateFrame("BUTTON", "Glance_petFrameClose", gfrm.petFrame, "UIPanelCloseButton")
		gfrm.petFrameClose:SetPoint("TOPRIGHT", gfrm.petFrame, "TOPRIGHT", -5, -6)	
		gfrm.petFrameClose:SetWidth(30)
		gfrm.petFrameClose:SetHeight(30)
						
		-- create the search box
		gfrm.petFrameSearch = CreateFrame("EDITBOX", "Glance_petFrameSearch", gfrm.petFrameFilter, "SearchBoxTemplate")
		gfrm.petFrameSearch:SetPoint("TOPLEFT", gfrm.petFrameFilter, "TOPRIGHT", 10, 5)	
		gfrm.petFrameSearch:SetWidth(gfrm.petFrame:GetWidth()-gfrm.petFrameFilter:GetWidth()-gfrm.petFrameClose:GetWidth()-25)
		gfrm.petFrameSearch:SetHeight(30)
		gfrm.petFrameSearch:SetScript("OnTextChanged", function(self, userInput) SearchBoxTemplate_OnTextChanged(self); gf.Pets.search(self) end)
		
		-------------------------------------------------------------
		
		-- create main model data container
		gfrm.petDataFrame = CreateFrame("FRAME", "Glance_petDataFrame", gfrm.petFrame, "TooltipBorderedFrameTemplate")
		gfrm.petDataFrame:SetPoint("TOPRIGHT", gfrm.petFrame, "TOPLEFT", 0, 0)	
		gfrm.petDataFrame:SetWidth(modelWidth)
		gfrm.petDataFrame:SetHeight(petHeight)
		gfrm.petDataFrame:Hide()
				
		-- create the title
		local title = gfrm.petDataFrame:CreateFontString("Glance_Pet_Model_Title")
		title:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+7)
		title:SetJustifyH("CENTER")
		title:SetJustifyV("CENTER")
		title:SetWordWrap(true)
		title:SetPoint("TOP", gfrm.petDataFrame, "TOP", 0, -5)
		title:SetTextColor(.9, .7, .2 ,1)
		title:SetHeight(20)
		title:SetWidth(modelWidth-40)		
		
		-- create the level
		local level = gfrm.petDataFrame:CreateFontString("Glance_Pet_Model_Level")
		level:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+9)
		level:SetJustifyH("CENTER")
		level:SetJustifyV("CENTER")
		level:SetWordWrap(true)
		level:SetPoint("TOPRIGHT", gfrm.petDataFrame, "TOPRIGHT", -10, -8)
		level:SetTextColor(.9, .7, .2 ,1)
		level:SetHeight(20)
		level:SetWidth(20)	
		
		-- set favorite icon
		gfrm.petDataFrame.favtexture = gfrm.petDataFrame:CreateTexture("Glance_petModelFavorite", "OVERLAY")
		gfrm.petDataFrame.favtexture:SetTexture(0,0,0,0)
		gfrm.petDataFrame.favtexture:SetPoint("TOPLEFT",10,-10)
		gfrm.petDataFrame.favtexture:SetWidth(16)
		
		-- create the notes
		local notes = gfrm.petDataFrame:CreateFontString("Glance_Pet_Model_Notes")
		notes:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+6)
		notes:SetJustifyH("LEFT")
		notes:SetJustifyV("BOTTOM")
		notes:SetPoint("BOTTOMLEFT", gfrm.petDataFrame, "BOTTOMLEFT", 10, 10)
		notes:SetTextColor(1, .46, 0 ,1)
		notes:SetWidth(modelWidth)
		notes:SetText("Left-Click to summon.\nRight-Click to set favorite.")
		
		-------------------------------------------------------------
		
		-- create model frame
		gfrm.petModelFrame = CreateFrame("PlayerModel","Glance_petModelFrame",gfrm.petFrame)
		gfrm.petModelFrame:SetPoint("TOP",gfrm.petDataFrame,"TOP",0,-25)
		gfrm.petModelFrame:SetHeight(modelWidth-100)
		gfrm.petModelFrame:SetWidth(modelWidth-10)
		gfrm.petModelFrame:SetPosition(0,0,0)
		gfrm.petModelFrame:SetAlpha(1)
		gfrm.petModelFrame:Hide()
		-- rotate the critter smoothly..
		gfrm.petModelFrame:SetScript("OnUpdate", function(self, elapsed) 
			seqtime = seqtime + elapsed; 
			if seqtime >= .01 then
				seqtime = 0
				position = position + .01
				if position <= 6.2 then
					gfrm.petModelFrame:SetFacing(position);
				else
					position = 0
					gfrm.petModelFrame:SetFacing(position);
				end
			end
		end)
		
		-- set type icon
		gfrm.petModelFrame.typetexture = gfrm.petModelFrame:CreateTexture("Glance_petModelType", "OVERLAY")
		gfrm.petModelFrame.typetexture:SetTexture(0,0,0,0)
		gfrm.petModelFrame.typetexture:SetPoint("BOTTOMLEFT",8,8)
		gfrm.petModelFrame.typetexture:SetHeight(32)
		gfrm.petModelFrame.typetexture:SetWidth(32)
		
		-- set health bar
		gfrm.petModelHealthBar = CreateFrame("StatusBar", "Glance_petModelHealthBar", gfrm.petModelFrame)
		gfrm.petModelHealthBar:SetPoint("TOPLEFT", gfrm.petModelFrame, "BOTTOMLEFT", 0, 0)
		gfrm.petModelHealthBar:SetWidth(gfrm.petModelFrame:GetWidth())
		gfrm.petModelHealthBar:SetHeight(5)
		gfrm.petModelHealthBar:SetMinMaxValues(0, 100)
		gfrm.petModelHealthBar:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
		gfrm.petModelHealthBar:GetStatusBarTexture():SetHorizTile(false)
		gfrm.petModelHealthBar:GetStatusBarTexture():SetVertTile(false)
		gfrm.petModelHealthBar:SetStatusBarColor(0, 0.65, 0)
		gfrm.petModelHealthBar.bg = gfrm.petModelHealthBar:CreateTexture(nil, "BACKGROUND")
		gfrm.petModelHealthBar.bg:SetTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
		gfrm.petModelHealthBar.bg:SetAllPoints(true)
		gfrm.petModelHealthBar.bg:SetVertexColor(0, 0.35, 0)
		
		-- set xp bar
		gfrm.petModelXPBar = CreateFrame("StatusBar", "Glance_petModelXPBar", gfrm.petModelHealthBar)
		gfrm.petModelXPBar:SetPoint("TOPLEFT", gfrm.petModelHealthBar, "BOTTOMLEFT", 0, -37)
		gfrm.petModelXPBar:SetWidth(gfrm.petModelFrame:GetWidth())
		gfrm.petModelXPBar:SetHeight(5)
		gfrm.petModelXPBar:SetMinMaxValues(0, 100)
		gfrm.petModelXPBar:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
		gfrm.petModelXPBar:GetStatusBarTexture():SetHorizTile(false)
		gfrm.petModelXPBar:GetStatusBarTexture():SetVertTile(false)
		gfrm.petModelXPBar:SetStatusBarColor(.4, .8, .94)
		gfrm.petModelXPBar.bg = gfrm.petModelXPBar:CreateTexture(nil, "BACKGROUND")
		gfrm.petModelXPBar.bg:SetTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
		gfrm.petModelXPBar.bg:SetAllPoints(true)
		gfrm.petModelXPBar.bg:SetVertexColor(0, 0, 0.65)
				
		-- create the specs
		local specs = gfrm.petModelHealthBar:CreateFontString("Glance_Pet_Model_Specs")
		specs:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+7)
		specs:SetJustifyH("CENTER")
		specs:SetJustifyV("TOP")
		specs:SetPoint("TOP", gfrm.petModelHealthBar, "BOTTOM", 5, -5)
		specs:SetTextColor(1, 1, 1 ,1)
		specs:SetWidth(modelWidth-15)
		
		-- create the description
		local description = gfrm.petModelXPBar:CreateFontString("Glance_Pet_Model_Description")
		description:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+6)
		description:SetJustifyH("LEFT")
		description:SetJustifyV("TOP")
		description:SetPoint("TOPLEFT", gfrm.petModelXPBar, "BOTTOMLEFT", 5, -5)
		description:SetTextColor(1, 1, 1 ,1)
		description:SetWidth(modelWidth-15)
		
	end
end

---------------------------
-- build buttons
---------------------------
function gf.Pets.buildButtons()
	Glance.Debug("function","buildButtons","Pets")
	-- variables
	local previousFrame = gfrm.petListFrame
	local numPets, totalPets = C_PetJournal.GetNumPets()
	-- counts do not match, update buttons
	if totalCount ~= totalPets then
		-- totalCount not zero, buttons have been built before, reset previous
		if totalCount > 0 then
			if _G["Glance_Pet_Buttons_"..totalCount] == nil then
				-- let's not throw an error checking for nil
			else
				-- this should be the last button
				previousFrame = _G["Glance_Pet_Buttons_"..totalCount]
			end
		end
		for i=1, totalPets do	
			-- if we have not yet created this button.
			if _G["Glance_Pet_Buttons_"..i] == nil then
				-- create the frame
				local m_btn = CreateFrame("BUTTON", "Glance_Pet_Buttons_"..i, gfrm.petListFrame)
				m_btn:SetHeight(iconSize)
				m_btn:SetWidth(250)
				
				-- create the font
				local fs = m_btn:CreateFontString("Glance_Pet_Buttons_FontString_"..i)
				fs:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+7)
				fs:SetJustifyH("LEFT")
				if IsAddOnLoaded("BattlePetBreedID") then
					fs:SetJustifyV("TOP")
				else
					fs:SetJustifyV("CENTER")
				end
				fs:SetPoint("TOPLEFT", 0, 0)
				fs:SetTextColor(1,1,1,1)
				
				-- create the icon overlay
				m_btn.usetexture = m_btn:CreateTexture("Glance_Pet_Buttons_UTexture_"..i, "BACKGROUND")
				m_btn.usetexture:SetTexture(0,0,0,0)
				m_btn.usetexture:SetPoint("TOPLEFT",-1,1)
				m_btn.usetexture:SetWidth(iconSize+3)
				m_btn.usetexture:SetHeight(iconSize+4)
				
				-- create the level font
				local lvlfs = m_btn:CreateFontString("Glance_Pet_Buttons_LevelFontString_"..i)
				lvlfs:SetFont("Fonts\\FRIZQT__.TTF", 9, "THICKOUTLINE")
				lvlfs:SetJustifyH("CENTER")
				lvlfs:SetJustifyV("CENTER")
				lvlfs:SetPoint("BOTTOMRIGHT", m_btn.usetexture, "BOTTOMRIGHT", 6, -2)
				lvlfs:SetTextColor(.9, .7, .2,1)
				
				-- create the breed font
				local brdfs = m_btn:CreateFontString("Glance_Pet_Buttons_BreedFontString_"..i)
				brdfs:SetFont(ga.Font[3][2], Glance_Local.Options.fontSize+5)
				brdfs:SetJustifyH("LEFT")
				brdfs:SetJustifyV("CENTER")
				brdfs:SetPoint("BOTTOMLEFT", m_btn.usetexture, "BOTTOMRIGHT", 4, 0)
				--brdfs:SetTextColor(.9, .7, .2,1)
				
				-- set favorite icon
				m_btn.favtexture = m_btn:CreateTexture("Glance_Pet_Buttons_FTexture_"..i, "OVERLAY")
				m_btn.favtexture:SetTexture(0,0,0,0)
				m_btn.favtexture:SetPoint("TOPLEFT",-5,5)
				m_btn.favtexture:SetWidth(16)
				
				-- create the text
				m_btn:SetFontString(fs)
				m_btn:SetText("Button"..i)
				
				-- position it
				if previousFrame == gfrm.petListFrame then
					m_btn:SetPoint("TOPLEFT", gfrm.petListFrame, "TOPLEFT", 5, -5)
				else
					m_btn:SetPoint("TOPLEFT", previousFrame, "BOTTOMLEFT", 0, -5)
				end
				
				--make this frame the previous frame
				previousFrame = _G["Glance_Pet_Buttons_"..i]	
				
				-- reset the width to actual width
				m_btn:SetWidth(fs:GetStringWidth())
				m_btn:SetHeight(iconSize+6)
				
				-- register for clicks
				m_btn:RegisterForClicks("AnyUp")					
				
				-- turn it on
				m_btn:Show()
			end
		end
		totalCount = totalPets
	end
end

---------------------------
-- return a list of linked objects
---------------------------
function gf.Pets.mapObjects(index)
	if index then
		local m_btn = _G["Glance_Pet_Buttons_"..index]
		if m_btn then
			m_btn.fs         = _G["Glance_Pet_Buttons_FontString_"..index]
			m_btn.level      = _G["Glance_Pet_Buttons_LevelFontString_"..index]
			m_btn.breed      = _G["Glance_Pet_Buttons_BreedFontString_"..index]
			m_btn.favtexture = _G["Glance_Pet_Buttons_FTexture_"..index]
			m_btn.usetexture = _G["Glance_Pet_Buttons_UTexture_"..index]			
			return m_btn
		else
			return nil
		end
	else
		local t = _G["Glance_Pet_Model_Title"]
		local d = _G["Glance_Pet_Model_Description"]
		local h = _G["Glance_petModelHealthBar"]
		local x = _G["Glance_petModelXPBar"]
		local f = _G["Glance_petModelFavorite"]
		local l = _G["Glance_Pet_Model_Level"]
		local s = _G["Glance_Pet_Model_Specs"]
		local pt = _G["Glance_petModelType"]
		return t, d, h, x, f, l, s, pt
	end
end

---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Pets")
end